using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;
using System.Windows.Media.Imaging;
using System.Windows.Controls;

namespace CSWpfDesktopEditionSample
{   
    public class CustomWorldMapKitWmsWpfOverlay : WorldMapKitWmsWpfOverlay
    {
        public CustomWorldMapKitWmsWpfOverlay()
            : base()
        {
            TemporaryCache = new InMemoryBitmapTileCache() { MaximumTilesCount = 300 };
            DrawnTile += new System.EventHandler<DrawnTileTileOverlayEventArgs>(CustomWorldMapKitOverlay_DrawnTile);
        }

        public InMemoryBitmapTileCache TemporaryCache { get; set; }

        protected override void DrawStretchTiles(RectangleShape targetExtent)
        {
            base.DrawStretchTiles(targetExtent);

            int zoomLevelIndex = MapArguments.GetSnappedZoomLevelIndex(targetExtent);
            int tempZoomLevelIndex = zoomLevelIndex - 1;

            double cacheRatio = 0;
            Collection<BitmapTile> cachedBitmapTiles = new Collection<BitmapTile>();
            while (tempZoomLevelIndex > 0 && zoomLevelIndex - tempZoomLevelIndex < 5)
            { 
                double tempScale = MapArguments.ZoomLevelScales[tempZoomLevelIndex];
                TemporaryCache.TileMatrix = new MapSuiteTileMatrix(tempScale, TileWidth, TileHeight, MapArguments.MapUnit);
                Collection<BitmapTile> bitmapTiles = TemporaryCache.GetTiles(targetExtent);
                int cachedCount = bitmapTiles.Count((t) => { return t != null && t.Bitmap != null; });
                double currentRatio = (double)cachedCount / (double)bitmapTiles.Count;
                if (currentRatio > cacheRatio)
                {
                    cacheRatio = currentRatio;
                    cachedBitmapTiles.Clear();
                    foreach (var bitmapTile in bitmapTiles)
                    {
                        if (bitmapTile != null && bitmapTile.Bitmap != null)
                        {
                            cachedBitmapTiles.Add(bitmapTile);
                        }
                    }
                }
                tempZoomLevelIndex--;
            }

            if (cacheRatio != 0 && cachedBitmapTiles.Count != 0)
            {
                foreach (var cachedBitmapTile in cachedBitmapTiles)
                {
                    double bitmapWidth = cachedBitmapTile.Bitmap.Width;
                    double bitmapHeight = cachedBitmapTile.Bitmap.Height;
                    double resolution = Math.Max(cachedBitmapTile.BoundingBox.Width / bitmapWidth, cachedBitmapTile.BoundingBox.Height / bitmapHeight);

                    Stream streamSource = new MemoryStream();
                    cachedBitmapTile.Bitmap.Save(streamSource, System.Drawing.Imaging.ImageFormat.Png);

                    BitmapImage imageSource = new BitmapImage();
                    imageSource.BeginInit();
                    imageSource.StreamSource = streamSource;
                    imageSource.EndInit();

                    ThinkGeo.MapSuite.WpfDesktopEdition.Tile tile = new ThinkGeo.MapSuite.WpfDesktopEdition.Tile();
                    tile.TargetExtent = cachedBitmapTile.BoundingBox;
                    tile.ZoomLevelIndex = MapArguments.GetSnappedZoomLevelIndex(cachedBitmapTile.Scale);
                    tile.BeginInit();
                    tile.ImageSource = imageSource;
                    tile.EndInit();

                    DrawStretchTile(tile, targetExtent, MapArguments.CurrentResolution);
                    Canvas drawingCanvas = (Canvas)OverlayCanvas.Children[1];
                    drawingCanvas.Children.Add(tile);
                }
            }
        }

        private void CustomWorldMapKitOverlay_DrawnTile(object sender, DrawnTileTileOverlayEventArgs e)
        {
            BitmapTile bitmapTile = new BitmapTile((System.Drawing.Bitmap)(((System.Drawing.Bitmap)e.GeoCanvas.NativeImage).Clone()), e.DrawnTile.TargetExtent, e.GeoCanvas.CurrentScale);
            TemporaryCache.TileMatrix.Scale = e.GeoCanvas.CurrentScale;
            if (TemporaryCache.GetTiles(e.DrawnTile.TargetExtent).Where(t => { return t.Bitmap != null; }).Count() == 0)
            {
                try
                {
                    TemporaryCache.SaveTile(bitmapTile);
                }
                catch { }
            }
        }

        private static void DrawStretchTile(ThinkGeo.MapSuite.WpfDesktopEdition.Tile tile, RectangleShape targetExtent, double currentResolution)
        {
            double newTileWidth = tile.TargetExtent.Width / currentResolution;
            double newTileHeight = tile.TargetExtent.Height / currentResolution;
            tile.Width = newTileWidth;
            tile.Height = newTileHeight;

            double offsetX = (tile.TargetExtent.UpperLeftPoint.X - targetExtent.UpperLeftPoint.X) / currentResolution;
            double offsetY = (targetExtent.UpperLeftPoint.Y - tile.TargetExtent.UpperLeftPoint.Y) / currentResolution;
            tile.SetValue(Canvas.LeftProperty, offsetX);
            tile.SetValue(Canvas.TopProperty, offsetY);
        }
    }
}